//////////////////////////////////////////////////////////////////////////
//	"fcv.cpp"
//////////////////////////////////////////////////////////////////////////

#include	"fcv.h"

Fcv::Fcv()
{
	this->verifyMode = VERIFY_OFF;
	this->filesize_m = 0;
}

TCHAR *	Fcv::GetErrMsg(int errcode)	// static
{
	TCHAR	*msgbuf;
	int		ret;

	ret = ::FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS
	    , NULL
	    , errcode
	    , MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT) // ̌
	    , (LPTSTR) &msgbuf
	    , 0
	    , NULL
	);
	return ret > 0 ? msgbuf : "";
}

TCHAR *	Fcv::FormatTime(TCHAR * buffer, time_t tm)
{
	struct tm	*lt;

	lt = ::localtime(&tm);
	_stprintf(buffer, "%04u.%02u.%02u %02u:%02u:%02u"
	  , lt->tm_year + 1900
	  , lt->tm_mon + 1
	  , lt->tm_mday
	  , lt->tm_hour
	  , lt->tm_min
	  , lt->tm_sec );
	return buffer;
}

TCHAR *	Fcv::FormatFileSize(TCHAR * buffer, FileSize size)
{
	_stprintf(buffer, TEXT("%I64d"), (__int64) size);
	return buffer;
}


int		main(int argc, TCHAR ** argv)
{
	const TCHAR					*writePath = NULL;
	Fcv							fcv;
	FileList					filelist;
	FileList::ListCreateParam	lcp;
	FcvTest						fcvtest;

	{
		int				i;

		for ( i = 1; i < argc; i++ )
		{
			const TCHAR	*s;
			int			n;

			s = argv[i];
			if ( *s == TEXT('-') || *s == TEXT('/') )
			{	// IvV
				const TCHAR	*t;

				++s;
				t = argv[i+1];
				if ( _tcsicmp(s, TEXT("verify")) == 0 )
				{
					fcv.verifyMode = Fcv::VERIFY_FILE;
				} else if ( _tcsicmp(s, TEXT("filesize")) == 0 && i + 1 < argc && *t != TEXT('-') && *t != TEXT('/') )
				{
					if ( (n = _ttoi(t)) > 0 )
					{
						fcv.filesize_m = n;
						i++;
					}
				} else if ( _tcsicmp(s, TEXT("maxDirLevel")) == 0 && i + 1 < argc && *t != TEXT('-') && *t != TEXT('/') )
				{
					if ( (n = _ttoi(t)) > 0 )
					{
						lcp.maxDirLevel = n;
						i++;
					}
				} else if ( _tcsicmp(s, TEXT("maxDirFiles")) == 0 && i + 1 < argc && *t != TEXT('-') && *t != TEXT('/') )
				{
					if ( (n = _ttoi(t)) > 0 )
					{
						lcp.maxDirFileNum = n;
						i++;
					}
				} else if ( _tcsicmp(s, TEXT("minFileSize")) == 0 && i + 1 < argc && *t != TEXT('-') && *t != TEXT('/') )
				{
					if ( (n = _ttoi(t)) > 0 )
					{
						lcp.minFileSize = n;
						i++;
					}
				} else if ( _tcsicmp(s, TEXT("maxFileSize")) == 0 && i + 1 < argc && *t != TEXT('-') && *t != TEXT('/') )
				{
					if ( (n = _ttoi(t)) > 0 )
					{
						lcp.maxFileSize = n;
						i++;
					}
				} else
				{
					_ftprintf(stderr, TEXT("Parameter error! (%s)\n"), s);
					return 1;
				}
			} else
			{
				if ( writePath == NULL )
					writePath = s;
			}
		}
	}
	if ( writePath == NULL )
	{
		_ftprintf(stderr, TEXT("Parameter error!\n"));
		return 1;
	}
	if ( fcv.filesize_m > 0 )
		lcp.totalFileSize = (FileSize)fcv.filesize_m * (FileSize)1024 * (FileSize)1024;
	if ( lcp.minFileSize > lcp.maxFileSize )
	{
		_ftprintf(stderr, TEXT("Min/Max Filesize error!\n"));
		return 1;
	}

	if ( filelist.CreateFileList(lcp) < 0 )
	{
		_ftprintf(stderr, TEXT("CreateFileList error!\n"));
		return 1;
	}
	fcvtest.Create(filelist, writePath);
	if ( fcv.verifyMode == Fcv::VERIFY_FILE )
		fcvtest.Verify(filelist, writePath);
	fcvtest.Remove();

	return 0;
}
