//////////////////////////////////////////////////////////////////////////
//	"filelist.h"
//------------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////////

#include	"fcv.h"
#include	<mbstring.h>

RandLib		FileList::rand;

FileData::FileData()
{
	this->filename = NULL;
	this->filelist = NULL;
}

FileData::~FileData()
{
	if ( this->filename != NULL )
		delete [] this->filename;
	if ( this->filelist != NULL )
		delete this->filelist;
}

int		FileData::SetFilename(const FileChar * name)
{
	size_t	len;

	if ( this->filename != NULL )
		delete [] this->filename;
	len = _tcslen(name) + 1;
	if ( (this->filename = new FileChar [len]) == NULL )
		return -1;
	_tcscpy(this->filename, name);
	return 0;
}


FileList::FileList()
{
	this->fileNum = 0;
	this->dirNum = 0;
}

FileList::~FileList()
{
	this->clear();
}

void	FileList::clear(void)
{
	iterator	it;
	FileData	*fdata;

	for ( it = this->list.begin(); it != this->list.end(); it++ )
	{
		if ( (fdata = *it) != NULL )
			delete fdata;
	}
	this->fileNum = 0;
	this->dirNum = 0;
}

void	FileList::push_back(FileData * fdata)
{
	if ( fdata->fileattr & FILE_ATTRIBUTE_DIRECTORY )
		this->dirNum++;
	else
		this->fileNum++;
	this->list.push_back(fdata);
}


int		FileList::ReadDir(const FileChar * dir, BOOL subdir)
{
	HANDLE				hFind;
	WIN32_FIND_DATA 	fi;

	if ( (hFind = ::FindFirstFile(dir, &fi)) == INVALID_HANDLE_VALUE )
		return 0;
	do
	{
		FileData	*fdata;

		if ( _tcscmp(fi.cFileName, TEXT(".")) == 0 || _tcscmp(fi.cFileName, TEXT("..")) == 0 )
			continue;
		if ( (fdata = new FileData) == NULL )
			break;	// error
		fdata->SetFilename(fi.cFileName);
		fdata->filesize  = ((FileSize)fi.nFileSizeHigh<<32) + (FileSize)fi.nFileSizeLow;
		fdata->fileattr  = fi.dwFileAttributes;
		fdata->timestamp = fi.ftLastWriteTime;
		this->push_back(fdata);
		if ( subdir && (fdata->fileattr & FILE_ATTRIBUTE_DIRECTORY) != 0 )
		{	// TufBNg
			FileChar *	dir2;

			if ( (dir2 = FileList::AddPath(dir, fdata->filename)) != NULL )
			{
				if ( (fdata->filelist = new FileList) != NULL )
					fdata->filelist->ReadDir(dir2, TRUE);
				delete [] dir2;
			}
		}
	} while ( ::FindNextFile(hFind, &fi) != 0 );
	::CloseHandle(hFind);
	return 0;
}


FileChar *	FileList::AddPath(const FileChar * base, const FileChar * name) // static
{
	size_t			base_len, name_len;
	FileChar		*path, *d;
	int				last_ch;
	const FileChar	*s;

	base_len = _tcslen(base);
	name_len = _tcslen(name);
	if ( (path = new FileChar [base_len + name_len + 4]) == NULL )
		return NULL;
	d = path;
	last_ch = 0;
	s = base;
	while ( *s )
	{
		if ( _ismbblead(s[0]) && _ismbbtrail(s[1]) )
		{
			last_ch = (s[0]<<8) + s[1];
			*d++ = *s++;
			*d++ = *s++;
		} else
		{
			last_ch = s[0];
			*d++ = *s++;
		}
	}
	if ( last_ch != TEXT('/') && last_ch != TEXT('\\') && last_ch != TEXT(':') )
		*d++ = TEXT('\\');
	_tcscpy(d, name);
	return path;
}


int		FileList::CreateFileList(ListCreateParam & lcp)
{
	unsigned	dirLevel = 0;
	unsigned	totalFileNum = 0;
	unsigned	totalDirNum = 0;
	FileSize	totalFileSize = 0;
	FileList *	filelist = this;

	while ( totalFileSize < lcp.totalFileSize )
	{
		FileSize	fsize;
		FileChar	name[MAX_PATH];
		FileData	*fdata;
	
		fsize = lcp.minFileSize + (((unsigned) rand.rand()) % (lcp.maxFileSize - lcp.minFileSize + 1));
		if ( totalFileSize + fsize >= lcp.totalFileSize )
			fsize = lcp.totalFileSize - totalFileSize;
		_stprintf(name, TEXT("%s%d"), lcp.filename, totalFileNum);
		fdata = new FileData;
		fdata->SetFilename(name);
		fdata->filesize = fsize;
		fdata->fileattr = 0;

		if ( filelist->GetFileNum() >= lcp.maxDirFileNum )
		{
			FileData	*fdata2;

			if ( dirLevel >= lcp.maxDirLevel )
			{
				filelist = this;
				dirLevel = 0;
			}

			_stprintf(name, TEXT("%s%d"), lcp.dirname, totalDirNum);
			fdata2 = new FileData;
			fdata2->SetFilename(name);
			fdata2->filesize = 0;
			fdata2->fileattr = FILE_ATTRIBUTE_DIRECTORY;
			fdata2->filelist = new FileList;
			filelist->push_back(fdata2);
			totalDirNum++;

			filelist = fdata2->filelist;
			++dirLevel;
		}
		filelist->push_back(fdata);
		totalFileNum++;
		totalFileSize += fdata->filesize;
	}
	return 0;
}


#if	0
int		PrintDir(const FileChar * dir, FileList & filelist)
{
	FileList::iterator	it;

	{
		char	tmp[256];
		gets(tmp);
	}
	for ( it = filelist.begin(); it != filelist.end(); it++ )
	{
		FileData	*fdata;
		FileChar	*path;

		fdata = *it;
		if ( (path = FileList::AddPath(dir, fdata->filename)) == NULL )
			break;	// error
		if ( (fdata->fileattr & FILE_ATTRIBUTE_DIRECTORY) )
		{
			PrintDir(path, *(fdata->filelist));
		} else
		{
			printf("%s\\%s (%u)\n", dir, fdata->filename, (unsigned)fdata->filesize);
		}
		delete [] path;
	}

	return 0;
}

int		main()
{
	FileList					filelist;
	FileList::ListCreateParam	lcp;

	printf("***** FileList Debug *****\n\n");
	filelist.CreateFileList(lcp);

	{
		PrintDir(TEXT(".\\"), filelist);
	}

	{
		char	tmp[256];
		gets(tmp);
	}
	return 0;
}
#endif
