// A straightforward C++ implementation of MT19937
// Haruhiko Okumura <okumura@matsusaka-u.ac.jp>
// Based on C source by Takuji Nishimura and Makoto Matsumoto
// (initialization improved 2002-01-26)
#ifndef	RAND_H_INCLUDED
#define	RAND_H_INCLUDED

class MersenneTwister
{
public:
	typedef unsigned long uint32;

private:
	static const int	N;
	static const int	M;
	static const uint32	MAT_A;
	static const uint32	UMASK;
	static const uint32	LMASK;
	int					mti;
	uint32				*mt;	// array for the state vector

	inline uint32 twist(uint32 u, uint32 v)
	{
		return (((u & UMASK) | (v & LMASK)) >> 1) ^ ((v & 1) * MAT_A);
	}
	void init(uint32 s);

public:
	MersenneTwister(uint32 s = 5489);
	MersenneTwister(uint32 init_key[], int key_length);
	virtual ~MersenneTwister();

	uint32 genrand_int32(void);

	// generates a random number on [0,0x7fffffff]-interval
	long genrand_int31(void) { return long(genrand_int32() >> 1); }

	// What follows are by Isaku Wada
	// generates a random number on [0,1]-real-interval
	double genrand_real1(void) { return genrand_int32() * (1.0 / 4294967295.0); }

	// generates a random number on [0,1)-real-interval
	double genrand_real2(void) { return genrand_int32() * (1.0/4294967296.0); }

	// generates a random number on (0,1)-real-interval
	double genrand_real3(void) { return (((double)genrand_int32()) + 0.5) * (1.0/4294967296.0); }

	// generates a random number on [0,1) with 53-bit resolution
	double genrand_res53(void)
	{ 
		uint32 a = genrand_int32() >> 5;
		uint32 b = genrand_int32() >> 6;
		return(a * 67108864.0 + b) * (1.0 / 9007199254740992.0);
	}
};

#endif
